@@ -44,7 +44,10 @@ setMethod(f = "draw_heatmap_body",
     pushViewport(viewport(name = paste(object@name, "heatmap_body", kr, kc, sep = "_"), ...))
 
     mat = object@matrix[row_order, column_order, drop = FALSE]
-    col_matrix = map_to_colors(object@matrix_color_mapping, mat)
+    oe = try(col_matrix <- map_to_colors(object@matrix_color_mapping, mat), silent = TRUE)
+    if(inherits(oe, "try-error")) {
+        col_matrix = matrix(NA, nrow = nrow(mat), ncol = ncol(mat))
+    }
 
     nc = ncol(mat)
     nr = nrow(mat)
@@ -89,6 +92,10 @@ setMethod(f = "draw_heatmap_body",
         heatmap_width_pt = max(1, ceiling(convertWidth(unit(1, "npc"), "bigpts", valueOnly = TRUE)))
         heatmap_height_pt = max(1, ceiling(convertHeight(unit(1, "npc"), "bigpts", valueOnly = TRUE)))
 
+        if(raster_quality < 1) raster_quality = 1
+        heatmap_width_pt = ceiling(heatmap_width_pt * raster_quality)
+        heatmap_height_pt = ceiling(heatmap_height_pt * raster_quality)
+
         matrix_is_resized = FALSE
         # resize on the matrix
         raster_resize_mat = object@heatmap_param$raster_resize_mat
@@ -129,17 +136,25 @@ setMethod(f = "draw_heatmap_body",
             temp_image_width = ceiling(max(heatmap_width_pt, 1))
             temp_image_height = ceiling(max(heatmap_height_pt, 1))
         } else {
-            if(is.character(raster_quality)) {
-                temp_image_width = ceiling(max(heatmap_width_pt, nc, 1))
-                temp_image_height = ceiling(max(heatmap_height_pt, nr, 1))
-            } else {
-                if(raster_quality < 1) raster_quality = 1
-                temp_image_width = ceiling(max(heatmap_width_pt*raster_quality, 1))
-                temp_image_height = ceiling(max(heatmap_height_pt*raster_quality, 1))
-            }
+            temp_image_width = ceiling(max(heatmap_width_pt, 1))
+            temp_image_height = ceiling(max(heatmap_height_pt, 1))
         }
-        do.call(device_fun, c(list(filename = temp_image, 
-            width = temp_image_width, height = temp_image_height), raster_device_param))
+        temp_image_width = as.integer(temp_image_width)
+        temp_image_height = as.integer(temp_image_height)
+
+        if(!is.na(ht_opt$raster_temp_image_max_width)) {
+            temp_image_width = min(temp_image_width, ht_opt$raster_temp_image_max_width)
+        }
+        if(!is.na(ht_opt$raster_temp_image_max_height)) {
+            temp_image_height = min(temp_image_height, ht_opt$raster_temp_image_max_height)
+        }
+
+        oe = try(do.call(device_fun, c(list(filename = temp_image, 
+            width = temp_image_width, height = temp_image_height), raster_device_param)))
+        if(inherits(oe, "try-error")) {
+            stop_wrap(qq("The size of the temporary image for rasterization is too huge (@{temp_image_width} x @{temp_image_height} px) that it is cannot be handled by the device function `@{device_info[1]}:@{raster_device}()`. Please reduce the maximal size of temporary image by setting proper values for `ht_opt$raster_temp_image_max_width` and `ht_opt$raster_temp_image_max_height`."))
+        }
+
         if(object@heatmap_param$verbose) {
             qqcat("saving into a temp image (.@{device_info[2]}) with size @{temp_image_width}x@{temp_image_height}px.\n")
         }
@@ -362,6 +377,7 @@ setMethod(f = "draw_dend",
     if(is.null(max_height)) {
         max_height = dend_heights(dend)
     }
+    if(max_height == 0) max_height = 1
 
     if(side %in% c("left", "right")) {
         xscale = c(0, max_height)
