@@ -151,21 +151,38 @@ HeatmapAnnotation = function(...,
 	n_anno = 0
 
 	#### check system calls ####
-
 	# HeatmapAnnotation is either called by `HeatmapAnnotation()` or by `rowAnnotation()`/`columnAnnotation()`
 	sc = sys.calls()
 	nsc = length(sc)
-	if(nsc == 1) {
+	if(nsc == 1) {  # HeatmapAnnotation(...)
 		scl = as.list(sc[[1]])
 		arg_list = scl[-1]
 	} else {
+		
 		scl = as.list(sc[[nsc-1]])
 		if(is.function(scl[[1]])) {
-			scl = as.list(sc[[nsc]])
-			arg_list = scl[-1]
-		} else if(any(as.character(scl[[1]]) %in% c("HeatmapAnnotation", "rowAnnotation", "columnAnnotation"))) {
+			if(identical(scl[[1]], pheatmap)) {
+				scl = as.list(sc[[nsc]])
+				arg_list = scl[-1]
+			} else if(identical(scl[[1]], heatmap)) {
+				scl = as.list(sc[[nsc]])
+				arg_list = scl[-1]
+			} else if(identical(scl[[1]], heatmap.2)) {
+				scl = as.list(sc[[nsc]])
+				arg_list = scl[-1]
+			} else {
+				# do.call(rowAnnotation, list(...))
+				# do.call(columnAnnotation, list(...))
+				arg_list = scl[-1]
+			}
+		} else if(any(as.character(scl[[1]]) %in% c("HeatmapAnnotation", "rowAnnotation", "columnAnnotation"))) { 
+			# columnAnnotation(...), rowAnnotation(...)
+			# do.call("columnAnnotation", list(...))
+			# do.call("rowAnnotation", list(...))
 			arg_list = scl[-1]
 		} else {
+			# do.call("HeatmapAnnotation", list(...))
+			# do.call(HeatmapAnnotation, list(...))
 			scl = as.list(sc[[nsc]])
 			arg_list = scl[-1]
 		}
@@ -203,8 +220,13 @@ HeatmapAnnotation = function(...,
 		if(nm %in% names(anno_arg_list)) {
 			anno_value_list[[nm]] = anno_arg_list[[nm]]
 		} else if(nm == "df") {
-			for(nm2 in colnames(df))
-			anno_value_list[[nm2]] = df[, nm2]
+			for(nm2 in colnames(df)) {
+				if(is.null(rownames(df))) {
+					anno_value_list[[nm2]] = df[, nm2]
+				} else {
+					anno_value_list[[nm2]] = structure(df[, nm2], names = rownames(df))
+				}
+			}
 		}
 	}
 
@@ -1058,12 +1080,16 @@ anno_type = function(ha) {
         for(nm in names(x2@anno_list)) {
         	x2@anno_list[[nm]] = x2@anno_list[[nm]][i]
         }
+        x2@anno_size = do.call(unit.c, lapply(x2@anno_list, size))
+        size(x2) = sum(x2@anno_size) + sum(x2@gap) - x2@gap[length(x2@gap)]
 
     } else if(nargs() == 2) { # ha[1:4]
     	x2 = x
         for(nm in names(x2@anno_list)) {
         	x2@anno_list[[nm]] = x2@anno_list[[nm]][i]
         }
+        x2@anno_size = do.call(unit.c, lapply(x2@anno_list, size))
+        size(x2) = sum(x2@anno_size) + sum(x2@gap) - x2@gap[length(x2@gap)]
 
     } else if (!missing(i) && !missing(j)) { # ha[1:4, "foo"]
     	x2 = x
@@ -1075,6 +1101,7 @@ anno_type = function(ha) {
         x2@gap = x@gap[j]
         x2@gap[length(x2@gap)] = unit(0, "mm")
 
+        x2@anno_size = do.call(unit.c, lapply(x2@anno_list, size))
         size(x2) = sum(x2@anno_size) + sum(x2@gap) - x2@gap[length(x2@gap)]
     }
     
