@@ -44,7 +44,7 @@
 #
 # -DENDROGRAM_PADDING space bewteen dendrograms and heatmap body.
 # -DIMNAME_PADDING space between row/column names and heatmap body.
-# -TITLE_PADDING space between row/column titles and heatmap body.
+# -TITLE_PADDING space between row/column titles and heatmap body. The value can have length of two which corresponds to the botton and top padding.
 # -COLUMN_ANNO_PADDING space between column annotations and heatmap body.
 # -ROW_ANNO_PADDING space between row annotations and heatmap body.
 # -HEATMAP_LEGEND_PADDING space between heatmap legends and heatmaps
@@ -201,7 +201,13 @@ ht_opt = setGlobalOptions(
 	"__export_image_size__" = list(
 		.visible = FALSE,
 		.value = FALSE
-	)
+	),
+	save_last = list(
+		.value = FALSE
+	),
+	"validate_names" = TRUE,
+	raster_temp_image_max_width = 30000,
+	raster_temp_image_max_height = 30000
 )
 
 
@@ -227,7 +233,7 @@ ht_global_opt = ht_opt
 .ENV$current_annotation_which = NULL
 .ENV$row_order = NULL
 .ENV$row_pos = NULL
-
+.ENV$last = NULL
 
 # DENDROGRAM_PADDING = unit(0.5, "mm")
 # DIMNAME_PADDING = unit(1, "mm")
