@@ -13,9 +13,12 @@
 # lgd@grob
 Legends = setClass("Legends",
     slots = list(
+    	name = "ANY",
         grob = "ANY",
         type = "character",
-        n = "numeric"
+        n = "numeric",
+        multiple = "numeric",
+        direction = "character"
     )
 )
 
@@ -48,6 +51,10 @@ Legends = function(...) {
 # -col_fun A color mapping function which is used to make a continuous legend. Use `circlize::colorRamp2` to
 #     generate the color mapping function. If ``at`` is missing, the breaks recorded in the color mapping function
 #      are used for ``at``.
+# -name Name of the legend, internally used.
+# -grob The legend body can be specified by a pre-constructed `grid::grob` object.
+# -break_dist A zooming factor to control relative distance of two neighbouring break values.The length
+#     of it should be ``length(at) - 1`` or a scalar. 
 # -nrow For legend which is represented as grids, ``nrow`` controls number of rows of the grids if the grids
 #      are arranged into multiple rows.
 # -ncol Similar as ``nrow``, ``ncol`` controls number of columns of the grids if the grids
@@ -56,11 +63,16 @@ Legends = function(...) {
 # -grid_height The height of legend grid. It can also control the height of the continuous legend if it is horizontal.
 # -grid_width The width of legend grid. It can also control the width of the continuous legend if it is vertical.
 # -gap If legend grids are put into multiple rows or columns, this controls the gap between neighbouring rows or columns, measured as a `grid::unit` object.
+# -column_gap The same as ``gap``.
+# -row_gap Space between legend rows.
 # -labels_gp Graphic parameters for labels.
 # -labels_rot Text rotation for labels. It should only be used for horizontal continuous legend.
 # -border Color of legend grid borders. It also works for the ticks in the continuous legend.
 # -background Background colors for the grids. It is used when points and lines are the legend graphics.
 # -type Type of legends. The value can be one of ``grid``, ``points``, ``lines`` and ``boxplot``.
+# -graphics Self-defined graphics for legends. The value should be a list of functions.
+#           Each function should accept four argumets: ``x`` and ``y``: positions of the legend grid (center point), ``w`` and ``h``: width and height
+#           of the legend grid.
 # -legend_gp Graphic parameters for the legend grids. You should control the filled color of the legend grids by ``gpar(fill = ...)``.
 # -pch Type of points if points are used as legend. Note you can use single-letter as pch, e.g. ``pch = 'A'``.
 #      There are three additional integers that are valid for ``pch``: 26 and 27 for single diagonal lines and 28 for double diagonal lines.
@@ -99,17 +111,20 @@ Legends = function(...) {
 # col_fun = colorRamp2(c(0, 0.5, 1), c("blue", "white", "red"))
 # lgd = Legend(col_fun = col_fun, title = "foo", at = c(0, 0.1, 0.15, 0.5, 0.9, 0.95, 1))
 # draw(lgd, test = "unequal interval breaks")
-Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALSE,
-	grid_height = unit(4, "mm"), grid_width = unit(4, "mm"), gap = unit(2, "mm"),
+Legend = function(at, labels = at, col_fun, name = NULL, grob = NULL,
+	break_dist = NULL, nrow = NULL, ncol = 1, by_row = FALSE,
+	grid_height = unit(4, "mm"), 
+	grid_width = unit(4, "mm"), 
+	gap = unit(2, "mm"), column_gap = gap, row_gap = unit(0, "mm"),
 	labels_gp = gpar(fontsize = 10), labels_rot = 0,
 	border = NULL, background = "#EEEEEE",
-	type = "grid", legend_gp = gpar(),
+	type = "grid", graphics = NULL, legend_gp = gpar(),
 	pch = 16, size = unit(2, "mm"),
 	legend_height = NULL, legend_width = NULL,
 	direction = c("vertical", "horizontal"),
 	title = "", title_gp = gpar(fontsize = 10, fontface = "bold"),
 	title_position = c("topleft", "topcenter", "leftcenter", "lefttop", "leftcenter-rot", "lefttop-rot"),
-	title_gap = unit(1.5, "mm")) {
+	title_gap = unit(2, "mm")) {
 
 	dev.null()
 	on.exit(dev.off2())
@@ -122,15 +137,20 @@ Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALS
 		labels_gp$fontsize = 10
 	}
 
+	labels_gp = check_gp(labels_gp)
+	title_gp = check_gp(title_gp)
+
 	# odevlist = dev.list()
 	direction = match.arg(direction)[1]
 	title_position = match.arg(title_position)[1]
 	title_padding = title_gap
-	if(missing(col_fun)) {
+	if(!is.null(grob)) {
+		legend_body = legend_body_from_grob(grob)
+	} else if(missing(col_fun)) {
 		if(is.null(border)) border = "white"
 		legend_body = discrete_legend_body(at = at, labels = labels, nrow = nrow, ncol = ncol,
-			grid_height = grid_height, grid_width = grid_width, gap = gap, labels_gp = labels_gp,
-			border = border, background = background, type = type, legend_gp = legend_gp,
+			grid_height = grid_height, grid_width = grid_width, gap = gap, row_gap = row_gap, column_gap = column_gap, labels_gp = labels_gp,
+			border = border, background = background, type = type, graphics = graphics, legend_gp = legend_gp,
 			pch = pch, size = size, by_row = by_row)
 	} else {
 		if(!missing(col_fun) && missing(at)) {
@@ -138,17 +158,22 @@ Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALS
 			if(is.null(breaks)) {
 				stop_wrap("You should provide `at` for color mapping function\n")
 			}
-		
-			le1 = grid.pretty(range(breaks))
-			le2 = pretty(breaks, n = 3)
-			if(abs(length(le1) - 5) < abs(length(le2) - 5)) {
-				at = le1
+			
+			if(is.null(break_dist)) {
+				le1 = grid.pretty(range(breaks))
+				le2 = pretty(breaks, n = 3)
+				if(abs(length(le1) - 5) < abs(length(le2) - 5)) {
+					at = le1
+				} else {
+					at = le2
+				}
 			} else {
-				at = le2
+				at = breaks
 			}
 		}
+
 		if(direction == "vertical") {
-			legend_body = vertical_continuous_legend_body(at = at, labels = labels, col_fun = col_fun,
+			legend_body = vertical_continuous_legend_body(at = at, labels = labels, col_fun = col_fun, break_dist = break_dist,
 				grid_height = grid_height, grid_width = grid_width, legend_height = legend_height,
 				labels_gp = labels_gp, border = border)
 		} else {
@@ -160,7 +185,7 @@ Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALS
 					legend_extension = title_width + title_padding
 				}
 			}
-			legend_body = horizontal_continuous_legend_body(at = at, labels = labels, col_fun = col_fun,
+			legend_body = horizontal_continuous_legend_body(at = at, labels = labels, col_fun = col_fun, break_dist = break_dist,
 				grid_height = grid_height, grid_width = grid_width, legend_width = legend_width,
 				labels_gp = labels_gp, labels_rot = labels_rot, border = border, legend_extension = legend_extension)
 		}
@@ -171,14 +196,19 @@ Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALS
 		object@grob = legend_body
 		object@type = "single_legend_no_title"
 		object@n = 1
+		object@multiple = 1
+		object@direction = "vertical"
 		return(object)
 	}
 	if(!inherits(title, c("expression", "call"))) {
+		if(grepl("^\\s+$", title)) title = ""
 		if(title == "") {
 			object = new("Legends")
 			object@grob = legend_body
 			object@type = "single_legend_no_title"
 			object@n = 1
+			object@multiple = 1
+			object@direction = "vertical"
 			return(object)
 		}
 	}
@@ -243,7 +273,15 @@ Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALS
 			total_width = title_width + title_padding + legend_width
 			total_height = legend_height
 			if(title_position == "lefttop") {
-				title_y = unit(1, "npc")
+				if(missing(col_fun)) {
+					title_y = unit(1, "npc") - convertY(grobY(legend_body$children[[1]], 270), "mm")
+				} else {
+					if(direction == "horizontal") {
+						title_y = unit(1, "npc") - convertHeight(grobHeight(legend_body$children[[2]])*0.5 - grobHeight(textGrob(title, gp = title_gp))*0.5, "mm")
+					} else {
+						title_y = unit(1, "npc")
+					}
+				}
 				title_just = c("left", "top")
 			} else {
 				title_y = unit(0.5, "npc")
@@ -255,7 +293,7 @@ Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALS
 			edit_vp_in_legend_grob(legend_body, x = unit(1, "npc"), y = unit(1, "npc"), 
 				valid.just = c(1, 1)),
 			vp = viewport(width = total_width, height = total_height),
-			cl = "Legend"
+			cl = "legend"
 		)
 		attr(gf, "width") = total_width
 		attr(gf, "height") = total_height
@@ -281,7 +319,7 @@ Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALS
 			textGrob(title, x = unit(0, "npc"), y = title_y, just = title_just, gp = title_gp, rot = 90),
 			edit_vp_in_legend_grob(legend_body, x = unit(1, "npc"), y = unit(1, "npc"), 
 				valid.just = c(1, 1)),
-			vp = viewport(width = total_width, height = total_height),
+			vp = viewport(width = total_width, height = total_height, gp = gpar(fontsize = 10, lineheight = 0.8)),
 			cl = "legend"
 		)
 		attr(gf, "width") = total_width
@@ -291,7 +329,10 @@ Legend = function(at, labels = at, col_fun, nrow = NULL, ncol = 1, by_row = FALS
 	object = new("Legends")
 	object@grob = gf
 	object@type = "single_legend"
+	object@name = name
 	object@n = 1
+	object@multiple = 1
+	object@direction = "vertical"
 	return(object)
 }
 
@@ -317,10 +358,11 @@ heightDetails.Legend = function(x) {
 
 # grids are arranged by rows or columns
 discrete_legend_body = function(at, labels = at, nrow = NULL, ncol = 1, by_row = TRUE,
-	grid_height = unit(4, "mm"), grid_width = unit(4, "mm"), gap = unit(2, "mm"),
+	grid_height = unit(4, "mm"), grid_width = unit(4, "mm"), 
+	gap = unit(2, "mm"), column_gap = gap, row_gap = unit(0, "mm"),
 	labels_gp = gpar(fontsize = 10),
 	border = "white", background = "#EEEEEE",
-	type = "grid", legend_gp = gpar(),
+	type = "grid", graphics = NULL, legend_gp = gpar(),
 	pch = 16, size = unit(2, "mm")) {
 
 	n_labels = length(labels)
@@ -334,62 +376,147 @@ discrete_legend_body = function(at, labels = at, nrow = NULL, ncol = 1, by_row =
 		ncol = 1
 	}
 	ncol = ifelse(ncol > n_labels, n_labels, ncol)
+	if(length(grid_height) == 1) grid_height = rep(grid_height, nrow)
+	if(length(grid_width) == 1) grid_width = rep(grid_width, ncol)
 
-	labels_mat = matrix(c(labels, rep("", nrow*ncol - n_labels)), nrow = nrow, ncol = ncol, byrow = by_row)
+	if(!is.null(graphics)) {
+		if(length(graphics) != n_labels) {
+			stop_wrap("Length of `graphics` should be the same as number of labels.")
+		}
+		if(!all(sapply(graphics, is.function))) {
+			stop_wrap("`graphics` should be a list of functions.")
+		}
+	}
+	if(length(graphics) == 0) graphics = NULL
+
+	if(inherits(labels, "expression")) {
+		labels_mat = matrix(c(labels, rep("__NA__", nrow*ncol - n_labels)), nrow = nrow, ncol = ncol, byrow = by_row)
+		labels_are_expression = TRUE
+	} else {
+		labels_mat = matrix(c(labels, rep(NA, nrow*ncol - n_labels)), nrow = nrow, ncol = ncol, byrow = by_row)
+		labels_are_expression = FALSE
+	}
 	index_mat = matrix(1:(nrow*ncol), nrow = nrow, ncol = ncol, byrow = by_row)
+	if(labels_are_expression) {
+		l_na = sapply(1:ncol(labels_mat), function(i) {
+			x = labels_mat[, i]
+			all(is.na.expression(x))
+		})
+	} else {
+		l_na = apply(labels_mat, 2, function(x) all(is.na(x)))
+	}
+	if(any(l_na)) {
+		message_wrap(qq("No legend element is put in the last @{sum(l_na)} column@{ifelse(sum(l_na) > 1, 's', '')} under `ncol = @{ncol}`, maybe you should set `by_row = TRUE`? Reset `ncol` to @{sum(!l_na)}."))
+		ncol = sum(!l_na)
+	}
+	if(labels_are_expression) {
+		l_na = sapply(1:nrow(labels_mat), function(i) {
+			x = labels_mat[i, ]
+			all(is.na.expression(x))
+		})
+	} else {
+		l_na = apply(labels_mat, 1, function(x) all(is.na(x)))
+	}
+	if(any(l_na)) {
+		message_wrap(qq("No legend element is put in the last @{sum(l_na)} row@{ifelse(sum(l_na) > 1, 's', '')} under `nrow = @{nrow}`, maybe you should set `by_row = FALSE`? Reset `nrow` to @{sum(!l_na)}."))
+		nrow = sum(!l_na)
+	}
 
 	labels_padding_left = unit(1, "mm")
-
 	## max width for each column in the legend
 	labels_max_width = NULL
 	for(i in 1:ncol) {
 		if(i == 1) {
 			labels_max_width = max(do.call("unit.c", lapply(labels_mat[, i], function(x) {
-					g = grobWidth(textGrob(x, gp = labels_gp))
-					if(i < ncol) {
-						g = g + gap
+					if(labels_are_expression) {
+						if(identical(as.character(x), "__NA__")) {
+							x = ""
+						}
 					}
-					g
+					grobWidth(textGrob(x, gp = labels_gp))
 				})))
 		} else {
 			labels_max_width = unit.c(labels_max_width, max(do.call("unit.c", lapply(labels_mat[, i], function(x) {
-					g = grobWidth(textGrob(x, gp = labels_gp))
-					if(i < ncol) {
-						g = g + gap
+					if(labels_are_expression) {
+						if(identical(as.character(x), "__NA__")) {
+							x = ""
+						}
 					}
-					g
+					grobWidth(textGrob(x, gp = labels_gp))
 				}))))
 		}
 	}
 	labels_max_width = convertWidth(labels_max_width, "mm")
 
+	row_height = NULL
+	for(i in 1:nrow) {
+		if(i == 1) {
+			row_height = max(do.call("unit.c", lapply(labels_mat[i, ], function(x) {
+					if(labels_are_expression) {
+						if(identical(as.character(x), "__NA__")) {
+							x = ""
+						}
+					}
+					grobHeight(textGrob(x, gp = labels_gp))
+				})))
+		} else {
+			row_height = unit.c(row_height, max(do.call("unit.c", lapply(labels_mat[i, ], function(x) {
+					if(labels_are_expression) {
+						if(identical(as.character(x), "__NA__")) {
+							x = ""
+						}
+					}
+					grobHeight(textGrob(x, gp = labels_gp))
+				}))))
+		}
+	}
+	row_height = convertWidth(row_height + unit(4, "points"), "mm") # 4pt is the margin of the text
+	row_height_no_gap = row_height
+	for(i in seq_along(row_height)) {
+		row_height_no_gap[i]= row_height[i] = max(row_height[i], grid_height[i])
+		if(i < length(row_height)) {
+			row_height[i] = row_height[i] + row_gap
+		}
+	}
+
+	row_height = convertHeight(row_height, "mm")
+
 	legend_gp = recycle_gp(legend_gp, n_labels)
 
-	legend_body_width = grid_width*ncol + labels_padding_left*ncol + sum(labels_max_width)
-	legend_body_height = nrow*(grid_height)
+	legend_body_width = sum(grid_width) + labels_padding_left*ncol + sum(labels_max_width) + column_gap*(ncol-1)
+	legend_body_height = sum(row_height)
 	legend_body_width = convertWidth(legend_body_width, "mm")
 	legend_body_height = convertHeight(legend_body_height, "mm")
 
 	# legend grid
 	gl = list()
+	previous_x = unit(0, "mm")
 	for(i in 1:ncol) {
-		if(inherits(labels_mat[1, 1], "expression")) {
-			index = index_mat[, i][sapply(labels_mat[, i], function(x) x) != ""]
+		if(labels_are_expression) {
+			index = index_mat[, i][!is.na.expression(labels_mat[, i])]
 		} else {
-			index = index_mat[, i][labels_mat[, i] != ""]
+			index = index_mat[, i][!is.na(labels_mat[, i])]
 		}
 		ni = length(index)
-		y = (0:(ni-1))*(grid_height)
-		y = legend_body_height - y
+		y = do.call("unit.c", lapply(1:ni, function(ind) {
+			if(ind == 1) {
+				unit(0, "points")
+			} else {
+				sum(row_height[1:(ind-1)])
+			}
+		}))
+		y = legend_body_height - y # from top
 
-		labels_x = grid_width*i + sum(labels_max_width[1:i]) + labels_padding_left*i - labels_max_width[i]
-		labels_y = y - grid_height*0.5
+		labels_x = previous_x + grid_width[i] + labels_padding_left
+		labels_y = y - (row_height[1:ni] - row_gap)*0.5
+		if(ni == nrow) labels_y[nrow] = y[nrow] - row_height[nrow]*0.5
 		labels_x = convertWidth(labels_x, "mm")
 		labels_y = convertHeight(labels_y, "mm")
 		gl = c(gl, list(
 			textGrob(labels[index], x = labels_x, y = labels_y, just = "left", gp = labels_gp)
 		))
 
+		######### graphics ############
 		# grid
 		sgd = subset_gp(legend_gp, index)
 		sgd2 = gpar()
@@ -398,51 +525,75 @@ discrete_legend_body = function(at, labels = at, nrow = NULL, ncol = 1, by_row =
 		} else {
 			sgd2$fill = background
 		}
-		sgd2$col = border
-
-		grid_x = grid_width*i + sum(labels_max_width[1:i]) + labels_padding_left*i - labels_max_width[i] - labels_padding_left - grid_width*0.5
-		grid_y = y - grid_height*0.5
+		if(identical(border, "asis")) {
+			sgd2$col = sgd2$fill	
+		} else {
+			sgd2$col = border
+		}
+		
+		grid_x = previous_x
+		grid_y = y - (row_height[1:ni] - row_gap)*0.5
+		if(ni == nrow) grid_y[nrow] = y[nrow] - row_height[nrow]*0.5
 		grid_x = convertWidth(grid_x, "mm")
 		grid_x = rep(grid_x, length(grid_y))
 		grid_y = convertHeight(grid_y, "mm")
 
-		gl = c(gl, list(
-			rectGrob(x = grid_x, y = grid_y, width = grid_width, height = grid_height, gp = sgd2)
-		))
+		if(is.null(graphics)) {
 
-		if(any(c("points", "p") %in% type)) {
-			if(length(pch) == 1) pch = rep(pch, n_labels)
-			if(length(size) == 1) size = rep(size, n_labels)
+			gl = c(gl, list(
+				rectGrob(x = grid_x, y = grid_y, width = grid_width[i], height = row_height_no_gap, gp = sgd2, just = "left")
+			))
 
-			if(is.character(pch)) {
-				gl = c(gl, list(
-					textGrob(pch[index], x = grid_x, y = grid_y, gp = subset_gp(legend_gp, index))
-				))
-			} else {
-				gl = c(gl, 
-					.pointsGrob_as_a_list(x = grid_x, y = grid_y, pch = pch[index], gp = subset_gp(legend_gp, index), size = size, width = grid_width, height = grid_height)
-				)
+			if(any(c("points", "p") %in% type)) {
+				if(length(pch) == 1) pch = rep(pch, n_labels)
+				if(length(size) == 1) size = rep(size, n_labels)
+
+				if(is.character(pch)) {
+					gl = c(gl, list(
+						textGrob(pch[index], x = grid_x + grid_width[i]*0.5, y = grid_y, gp = subset_gp(legend_gp, index))
+					))
+				} else {
+					gl = c(gl, 
+						.pointsGrob_as_a_list(x = grid_x + grid_width[i]*0.5, y = grid_y, pch = pch[index], 
+							gp = subset_gp(legend_gp, index), size = size[index], width = grid_width[i], height = row_height_no_gap)
+					)
+				}
 			}
+			if(any(c("lines", "l") %in% type)) {
+				gl = c(gl, list(
+					segmentsGrob(x0 = grid_x, y0 = grid_y, 
+						         x1 = grid_x + grid_width[i], y1 = grid_y,
+						         gp = subset_gp(legend_gp, index))
+				))
+			}
+			if(any(c("boxplot", "box") %in% type)) {
+				gl = c(gl, list(
+					segmentsGrob(x0 = grid_x + grid_width[i]*0.5, y0 = grid_y - row_height_no_gap*0.45, 
+						         x1 = grid_x + grid_width[i]*0.5, y1 = grid_y + row_height_no_gap*0.45,
+						         gp = subset_gp(legend_gp, index)),
+					rectGrob(x = grid_x + grid_width[i]*0.5, y = grid_y, width = grid_width[i]*0.9, height = row_height_no_gap*0.5,
+						     gp = subset_gp(legend_gp, index)),
+					segmentsGrob(x0 = grid_x + grid_width[i]*0.05, y0 = grid_y, 
+						         x1 = grid_x + grid_width[i]*0.95, y1 = grid_y,
+						         gp = subset_gp(legend_gp, index))
+				))
+			}
+		} else {
+			fl = graphics[index]
+			gb_lt = list()
+			for(k in seq_along(fl)) {
+				gb_lt[[k]] = grid.grabExpr({
+						fl[[k]](x = grid_x[1] + grid_width[i]*0.5, y = grid_y[k], w = grid_width[i], h = row_height_no_gap[k])
+					}, 
+					width = convertWidth(grid_width[i], "inch", valueOnly = TRUE), 
+					height = convertHeight(row_height_no_gap[k], "inch", valueOnly = TRUE)
+				)
+					
+			}
+			gl = c(gl, gb_lt)
 		}
-		if(any(c("lines", "l") %in% type)) {
-			gl = c(gl, list(
-				segmentsGrob(x0 = grid_x - grid_width*0.5, y0 = grid_y, 
-					         x1 = grid_x + grid_width*0.5, y1 = grid_y,
-					         gp = subset_gp(legend_gp, index))
-			))
-		}
-		if(any(c("boxplot", "box") %in% type)) {
-			gl = c(gl, list(
-				segmentsGrob(x0 = grid_x, y0 = grid_y - grid_height*0.45, 
-					         x1 = grid_x, y1 = grid_y + grid_height*0.45,
-					         gp = subset_gp(legend_gp, index)),
-				rectGrob(x = grid_x, y = grid_y, width = grid_width*0.9, height = grid_height*0.5,
-					     gp = subset_gp(legend_gp, index)),
-				segmentsGrob(x0 = grid_x - grid_width*0.45, y0 = grid_y, 
-					         x1 = grid_x + grid_width*0.45, y1 = grid_y,
-					         gp = subset_gp(legend_gp, index))
-			))
-		}
+
+		previous_x = previous_x + grid_width[i] + labels_max_width[i] + labels_padding_left + column_gap
 	}
 
 	class(gl) = "gList"
@@ -453,26 +604,95 @@ discrete_legend_body = function(at, labels = at, nrow = NULL, ncol = 1, by_row =
 }
 
 vertical_continuous_legend_body = function(at, labels = at, col_fun,
-	grid_height = unit(4, "mm"), grid_width = unit(4, "mm"),
+	break_dist = NULL, grid_height = unit(4, "mm"), grid_width = unit(4, "mm"),
 	legend_height = NULL,
 	labels_gp = gpar(fontsize = 10),
 	border = NULL) {
 
-	od = order(at)
-	at = at[od]
-	labels = labels[od]
+	n = length(at)
+	breaks = attr(col_fun, "breaks")
+	if(identical(order(at), seq(n, 1))) {
+		breaks = rev(breaks)
+		break_dist = rev(break_dist)
+	} else {
+		od = order(at)
+		if(!is.null(break_dist)) {
+			if(!identical(od, 1:n)) {
+				stop_wrap("`at` should be sorted if `break_dist` is set.")
+			}
+		}
+		at = at[od]
+		labels = labels[od]
+	}
 
+	if(!is.null(break_dist)) {
+		if(is.null(breaks)) {
+			stop_wrap("`col_fun` must have a 'breaks' attribute if `break_dist` is set.")
+		}
+		if(length(break_dist) == 1) {
+			break_dist = rep(break_dist, n - 1)
+		}
+		if(length(break_dist) != n - 1) {
+			stop_wrap("Length of `break_dist` should be `length(at) - 1`.")
+		}
+		# this function is not vectorized
+		generate_map_fun = function(breaks) {
+			nb = length(breaks)
+			y = c(0, break_dist)
+			y = cumsum(y)/sum(y)
+			function(x) {
+				if(x <= y[1]) {
+					return(breaks[1])
+				} else if(x >= y[nb]) {
+					return(breaks[nb])
+				} else {
+					for(i in 2:nb) {
+						if(x == y[i]) {
+							return(breaks[i])
+						}
+						if(x < y[i]) {
+							return( (x - y[i-1])/(y[i] - y[i-1])*(breaks[i] - breaks[i-1]) + breaks[i-1] )
+						}
+					}
+				}
+			}
+		}
+		col_fun2 = col_fun
+		map_fun = generate_map_fun(sort(at))
+		col_fun = function(x) {
+			x2 = sapply(x, map_fun)
+			col_fun2(x2)
+		}
+		y = c(0, break_dist)
+		y = cumsum(y)/sum(y)
+
+		attr(col_fun, "breaks") = y
+		if(at[1] <= at[n]) {
+			at = y
+		} else {
+			at = rev(y)
+		}
+	}
+	
 	n_labels = length(labels)
 	labels_max_width = max_text_width(labels, gp = labels_gp)
 
 	labels_padding_left = unit(1, "mm")
 
-	min_legend_height = length(at)*(grid_height)
-	if(is.null(legend_height)) legend_height = min_legend_height
-	if(convertHeight(legend_height, "mm", valueOnly = TRUE) < convertHeight(min_legend_height, "mm", valueOnly = TRUE)) {
-		warning_wrap("`legend_height` you specified is too small, use the default minimal height.")
-		legend_height = min_legend_height
+	labels_gp = recycle_gp(labels_gp, n)
+	min_legend_height = unit(sum(sapply(seq_len(n), function(i) {
+		convertHeight(grobHeight(textGrob(labels[i], gp = subset_gp(labels_gp, i))), "mm", valueOnly = TRUE)
+	})), "mm") + unit(6 * (n-1), "points")
+
+	if(!is.null(legend_height)) {
+		if(convertHeight(legend_height, "mm", valueOnly = TRUE) < convertHeight(min_legend_height, "mm", valueOnly = TRUE)) {
+			warning_wrap("`legend_height` you specified is too small, use the default minimal height.")
+			legend_height = min_legend_height
+		}
+	} else {
+		legend_height = max(min_legend_height, unit(21.1, "mm")) # five-label legend
 	}
+	legend_height = convertHeight(legend_height, "mm")
 
 	segment_col = border
 	if(is_diff_equal(at)) {
@@ -492,9 +712,9 @@ vertical_continuous_legend_body = function(at, labels = at, col_fun,
 	gl = list()
 
 	# labels
-	labels_height = convertHeight(grobHeight(textGrob("foo", gp = labels_gp)), "mm")
+	labels_line_height = convertHeight(grobHeight(textGrob("fooy", gp = labels_gp)), "mm")
 	x = unit(rep(0, n_labels), "npc")
-	offset = unit(0.5, "mm")
+	offset = unit(2, "points") # space from the first break to the bottom and the last break to the top
 	k = length(at)
 	ymin = offset
 	ymax = legend_height-offset
@@ -504,13 +724,14 @@ vertical_continuous_legend_body = function(at, labels = at, col_fun,
 	labels_y = convertHeight(y, "mm")
 	
 	if(!at_diff_is_equal) {
-		labels_height = do.call("unit.c", lapply(labels, 
-			function(x) grobHeight(textGrob(x, gp = labels_gp)) + unit(2, "mm")))
-		y_top = labels_y + labels_height*0.5
-		y_bottom = labels_y - labels_height*0.5
+		labels_line_height = do.call("unit.c", lapply(labels, 
+			function(x) grobHeight(textGrob(x, gp = labels_gp)) + unit(4, "pt")))
+		y_top = labels_y + labels_line_height*0.5
+		y_bottom = labels_y - labels_line_height*0.5
 		y_top = convertY(y_top, "mm", valueOnly = TRUE)
 		y_bottom = convertY(y_bottom, "mm", valueOnly = TRUE)
-		yrange = c(0, convertHeight(legend_body_height, "mm", valueOnly = TRUE))
+		yrange = convertY(unit.c(offset - labels_line_height[1]*0.5, 
+			                     legend_body_height - offset + labels_line_height[length(labels_line_height)]*0.5), "mm", valueOnly = TRUE)
 		new_pos = smartAlign(y_bottom, y_top, yrange)
 		y2 = (new_pos[, 1] + new_pos[, 2])/2
 		y2 = unit(y2, "mm")
@@ -534,16 +755,22 @@ vertical_continuous_legend_body = function(at, labels = at, col_fun,
 	
 	## colors
 	at2 = unlist(lapply(seq_len(n_labels - 1), function(i) {
-		x = seq(at[i], at[i+1], length = round((at[i+1]-at[i])/(at[k]-at[1])*100))
+		x = seq(at[i], at[i+1], length.out = round((at[i+1]-at[i])/(at[k]-at[1])*100))
 		x = x[-length(x)]
 	}))
 	at2 = c(at2, at[length(at)])
 	colors = col_fun(at2)
 	x2 = unit(rep(0, length(colors)), "npc")
-	y2 = seq(0, 1, length = length(colors)+1)
-	y2 = y2[-length(y2)] * legend_body_height
+	y2 = seq(0, 1, length.out = length(colors)+1); y2 = y2[-length(y2)]
+	y2 = y2 * (legend_body_height - 2*offset) + offset
+	y2 = y2 + (y2[2] - y2[1])*0.5
+	hh = (legend_body_height - 2*offset)*(1/length(colors))
+	x2 = unit.c(unit(0, "npc"), x2, unit(0, "npc"))
+	y2 = unit.c(offset*0.5, y2, legend_body_height - offset*0.5)
+	hh = unit.c(offset, rep(hh, length(y2)-2), offset)
+	colors = c(colors[1], colors, colors[length(colors)])
 	gl = c(gl, list(
-		rectGrob(x2, rev(y2), width = grid_width, height = (unit(1, "npc"))*(1/length(colors)), just = c("left", "center"),
+		rectGrob(x2, rev(y2), width = grid_width, height = hh, just = c("left", "center"),
 			gp = gpar(col = rev(colors), fill = rev(colors))),
 		segmentsGrob(unit(0, "npc"), y, unit(0.8, "mm"), y, gp = gpar(col = ifelse(is.null(border), "white", border))),
 		segmentsGrob(grid_width, y, grid_width - unit(0.8, "mm"), y, gp = gpar(col = ifelse(is.null(border), "white", border)))
@@ -587,15 +814,75 @@ vertical_continuous_legend_body = function(at, labels = at, col_fun,
 }
 
 horizontal_continuous_legend_body = function(at, labels = at, col_fun,
-	grid_height = unit(4, "mm"), grid_width = unit(4, "mm"),
+	break_dist = NULL, grid_height = unit(4, "mm"), grid_width = unit(4, "mm"),
 	legend_width = NULL,
 	labels_gp = gpar(fontsize = 10), labels_rot = 0,
 	border = NULL, legend_extension = unit(0, "mm")) {
 		
-	od = order(at)
-	at = at[od]
-	labels = labels[od]
-	k = length(at)
+	n = k = length(at)
+	breaks = attr(col_fun, "breaks")
+	if(identical(order(at), seq(k, 1))) {
+		breaks = rev(breaks)
+		break_dist = rev(break_dist)
+	} else {
+		od = order(at)
+		if(!is.null(break_dist)) {
+			if(!identical(od, 1:n)) {
+				stop_wrap("`at` should be sorted if `break_dist` is set.")
+			}
+		}
+		at = at[od]
+		labels = labels[od]
+	}
+
+	if(!is.null(break_dist)) {
+		if(is.null(breaks)) {
+			stop_wrap("`col_fun` must have a 'breaks' attribute if `break_dist` is set.")
+		}
+		if(length(break_dist) == 1) {
+			break_dist = rep(break_dist, n - 1)
+		}
+		if(length(break_dist) != n - 1) {
+			stop_wrap(qq("Length of `break_dist` should be `length(at) - 1` which is @{length(at) - 1}."))
+		}
+		# this function is not vectorized
+		generate_map_fun = function(breaks) {
+			nb = length(breaks)
+			y = c(0, break_dist)
+			y = cumsum(y)/sum(y)
+			function(x) {
+				if(x <= y[1]) {
+					return(breaks[1])
+				} else if(x >= y[nb]) {
+					return(breaks[nb])
+				} else {
+					for(i in 2:nb) {
+						if(x == y[i]) {
+							return(breaks[i])
+						}
+						if(x < y[i]) {
+							return( (x - y[i-1])/(y[i] - y[i-1])*(breaks[i] - breaks[i-1]) + breaks[i-1] )
+						}
+					}
+				}
+			}
+		}
+		col_fun2 = col_fun
+		map_fun = generate_map_fun(sort(at))
+		col_fun = function(x) {
+			x2 = sapply(x, map_fun)
+			col_fun2(x2)
+		}
+		y = c(0, break_dist)
+		y = cumsum(y)/sum(y)
+
+		attr(col_fun, "breaks") = y
+		if(at[1] <= at[n]) {
+			at = y
+		} else {
+			at = rev(y)
+		}
+	}
 
 	labels_rot = labels_rot %% 360
 
@@ -610,7 +897,7 @@ horizontal_continuous_legend_body = function(at, labels = at, col_fun,
 
 	labels_padding_top = unit(1, "mm")
 
-	min_legend_width = sum(labels_width)*1.5
+	min_legend_width = sum(labels_width) + unit(2.1, "mm")*n_labels
 	if(is.null(legend_width)) legend_width = min_legend_width
 
 	segment_col = border
@@ -623,11 +910,8 @@ horizontal_continuous_legend_body = function(at, labels = at, col_fun,
 	gl = list()
 
 	# legend grid
-	if(labels_rot != 0) {
-		offset = convertHeight(grobHeight(textGrob("foo", gp = labels_gp))*0.5, "mm")
-	} else {
-		offset = unit(0.5, "mm")
-	}
+	offset = unit(2, "pt")
+	
 	xmin = offset
 	xmax = legend_body_width - offset
 	x = (at - at[1])/(at[k] - at[1])*(xmax - xmin)+ xmin
@@ -642,30 +926,20 @@ horizontal_continuous_legend_body = function(at, labels = at, col_fun,
 		labels_just = "left"
 	}
 	# adjust the text position
-	if(labels_rot == 0) {
-		labels_width = do.call("unit.c", lapply(labels, 
-			function(x) grobWidth(textGrob(x, gp = labels_gp))))
-		x_right = labels_x + labels_width*0.5
-		x_left = labels_x - labels_width*0.5
-		x_right = convertX(x_right, "mm", valueOnly = TRUE)
-		x_left = convertX(x_left, "mm", valueOnly = TRUE)
-	} else {
-		labels_height = do.call("unit.c", lapply(labels, 
-			function(x) grobHeight(textGrob(x, gp = labels_gp))))
-		x_right = labels_x + labels_height*0.5
-		x_left = labels_x - labels_height*0.5
-		x_right = convertX(x_right, "mm", valueOnly = TRUE)
-		x_left = convertX(x_left, "mm", valueOnly = TRUE)
-	}
-	ext = convertX(legend_extension, "mm", valueOnly = TRUE)
-	ext = max(convertWidth(grobWidth(textGrob(labels[1], gp = labels_gp))*0.5, "mm", valueOnly = TRUE), ext)
-	if(labels_rot == 0) {
-		xrange = c(0, ext + convertWidth(legend_body_width, "mm", valueOnly = TRUE) + convertWidth(grobWidth(textGrob(labels[n_labels], gp = labels_gp))*0.5, "mm", valueOnly = TRUE))
-	} else {
-		xrange = c(0, ext + convertWidth(legend_body_width, "mm", valueOnly = TRUE) + convertHeight(grobHeight(textGrob(labels[n_labels], gp = labels_gp))*0.5, "mm", valueOnly = TRUE))
-	}
+	labels_width = do.call("unit.c", lapply(labels, 
+		function(x) grobWidth(textGrob(x, gp = labels_gp, rot = labels_rot)) + unit(1, "mm")))
+	x_right = labels_x + labels_width*0.5
+	x_left = labels_x - labels_width*0.5
+	x_right = convertX(x_right, "mm", valueOnly = TRUE)
+	x_left = convertX(x_left, "mm", valueOnly = TRUE)
 
-	new_pos = smartAlign(x_left + ext, x_right + ext, xrange) - ext
+	# adjust to the left extension, caused by e.g. title
+	ext = max(legend_extension, labels_width[1]*0.5 - offset)
+	xrange = convertX(unit.c(-1*ext,
+			legend_body_width - offset + labels_width[length(labels_width)]*0.5
+	), "mm", valueOnly = TRUE)
+
+	new_pos = smartAlign(x_left, x_right, xrange)
 	x2 = (new_pos[, 1] + new_pos[, 2])/2
 	x2 = unit(x2, "mm")
 	labels_x = x2
@@ -673,22 +947,7 @@ horizontal_continuous_legend_body = function(at, labels = at, col_fun,
 	if(all(abs(as.numeric(labels_x) - as.numeric(x)) < 1e-4)) {
 		adjust_text_pos = FALSE
 	} else {
-		# recalculate with adding 2mm padding between labels
-		if(labels_rot == 0) {
-			labels_width = do.call("unit.c", lapply(labels, 
-			function(x) grobWidth(textGrob(x, gp = labels_gp)) + unit(2, "mm")))
-			x_right = labels_x + labels_width*0.5
-			x_left = labels_x - labels_width*0.5
-			x_right = convertX(x_right, "mm", valueOnly = TRUE)
-			x_left = convertX(x_left, "mm", valueOnly = TRUE)
-			
-			xrange = c(0, ext + convertWidth(legend_body_width, "mm", valueOnly = TRUE) + convertWidth(grobWidth(textGrob(labels[n_labels], gp = labels_gp))*0.5, "mm", valueOnly = TRUE) + 1)
-			new_pos = smartAlign(x_left + ext, x_right + ext, xrange) - ext
-			x2 = (new_pos[, 1] + new_pos[, 2])/2
-			x2 = unit(x2, "mm")
-			labels_x = x2
-		}
-
+		
 		adjust_text_pos = TRUE
 		labels_padding_top = unit(4, "mm")
 		legend_body_height = grid_height + labels_padding_top + labels_max_height
@@ -701,17 +960,23 @@ horizontal_continuous_legend_body = function(at, labels = at, col_fun,
 	))
 
 	at2 = unlist(lapply(seq_len(n_labels - 1), function(i) {
-		x = seq(at[i], at[i+1], length = round((at[i+1]-at[i])/(at[k]-at[1])*100))
+		x = seq(at[i], at[i+1], length.out = round((at[i+1]-at[i])/(at[k]-at[1])*100))
 		x = x[-length(x)]
 	}))
 	at2 = c(at2, at[length(at)])
 	colors = col_fun(at2)
 	y2 = unit(rep(1, length(colors)), "npc")
-	x2 = seq(0, 1, length = length(colors)+1)
-	x2 = x2[-length(x2)] * legend_body_width
-	
+	x2 = seq(0, 1, length.out = length(colors)+1); x2 = x2[-length(x2)] # left of rects
+	x2 = x2 * (legend_body_width - 2*offset) + offset
+	x2 = x2 + (x2[2] - x2[1])*0.5
+	ww = (legend_body_width - 2*offset)*(1/length(colors))
+	y2 = unit.c(unit(1, "npc"), y2)
+	x2 = unit.c(unit(0, "npc"), x2)
+	ww = unit.c(offset + (x2[2] - x2[1])*0.5, rep(ww, length(x2) - 2))
+	colors = c(colors[1], colors)
+
 	gl = c(gl, list(
-		rectGrob(x2, y2, height = grid_height, width = (unit(1, "npc"))*(1/length(colors)), just = c("left", "top"),
+		rectGrob(x2, y2, height = grid_height, width = ww, just = c("left", "top"),
 			gp = gpar(col = colors, fill = colors)),
 		segmentsGrob(x, legend_body_height - grid_height, x, legend_body_height - grid_height + unit(0.8, "mm"), gp = gpar(col = ifelse(is.null(border), "white", border))),
 		segmentsGrob(x, legend_body_height - unit(0.8, "mm"), x, legend_body_height, gp = gpar(col = ifelse(is.null(border), "white", border)))
@@ -754,6 +1019,20 @@ horizontal_continuous_legend_body = function(at, labels = at, col_fun,
 	return(gt)
 }
 
+legend_body_from_grob = function(grob) {
+	legend_body_width = grobWidth(grob)
+	legend_body_height = grobHeight(grob)
+	grob$x = unit(0, "npc")
+	grob$y = unit(0, "npc")
+	grob$just =
+	gl = list(grob)
+	class(gl) = "gList"
+	gt = gTree(children = gl, cl = "legend_body", vp = viewport(width = legend_body_width, height = legend_body_height))
+	attr(gt, "height") = legend_body_height
+	attr(gt, "width") = legend_body_width
+	return(gt)
+}
+
 # == title
 # Pack Legends
 #
@@ -785,7 +1064,7 @@ horizontal_continuous_legend_body = function(at, labels = at, col_fun,
 # draw(pd, test = "two legends")
 # pd = packLegend(lgd1, lgd2, direction = "horizontal")
 # draw(pd, test = "two legends packed horizontally")
-packLegend = function(..., gap = unit(2, "mm"), row_gap = unit(2, "mm"), column_gap = unit(2, "mm"),
+packLegend = function(..., gap = unit(4, "mm"), row_gap = unit(4, "mm"), column_gap = unit(4, "mm"),
 	direction = c("vertical", "horizontal"),
 	max_width = NULL, max_height = NULL, list = NULL) {
 
@@ -824,6 +1103,8 @@ packLegend = function(..., gap = unit(2, "mm"), row_gap = unit(2, "mm"), column_
 		stop_wrap("Length of `column_gap` must be one.")
 	}
     n_lgd = length(legend_list)
+    nr = 1
+    nc = 1
     if(direction == "vertical") {
 	    lgd_height = do.call("unit.c", lapply(legend_list, grobHeight))
 
@@ -947,7 +1228,13 @@ packLegend = function(..., gap = unit(2, "mm"), row_gap = unit(2, "mm"), column_
 	object = new("Legends")
 	object@grob = gt
 	object@type = "packed_legends"
+	object@direction = direction
 	object@n = n_lgd
+	if(direction == "vertical") {
+		object@multiple = nc
+	} else {
+		object@multiple = nr
+	}
 	return(object)
 }
 
@@ -992,12 +1279,26 @@ edit_vp_in_legend_grob = function(gtree, ...) {
 		gtree$vp = do.call(viewport, vp_param)
 		if(!is.null(valid.just)) {
 			gtree$vp$valid.just = valid.just
+			gtree$vp$justification = valid.just
 		}
 	} else {
 		vp_param = list(...)
 		nm = names(vp_param)
 		if("just" %in% nm) {
-			vp_param$valid.just = valid_just(vp_param$just)
+			if(is.numeric(vp_param$just)) {
+				vp_param$valid.just = vp_param$just
+			} else {
+				vp_param$valid.just = valid_just(vp_param$just)
+			}
+		}
+		if("justification" %in% nm) {
+			if(is.numeric(vp_param$justification)) {
+				vp_param$valid.just = vp_param$justification
+			} else {
+				vp_param$valid.just = valid_just(vp_param$justification)
+			}
+		} else {
+			vp_param$justification = vp_param$valid.just
 		}
 		
 		for(nm in names(vp_param)) {
@@ -1011,6 +1312,7 @@ edit_vp_in_legend_grob = function(gtree, ...) {
 }
 
 valid_just = function(just) {
+	if(is.numeric(just)) return(just)
 	if(length(just) == 1) {
 		just = switch(just,
 			"centre" = c("center", "center"),
@@ -1066,7 +1368,7 @@ valid_just = function(just) {
 setMethod(f = "draw",
 	signature = "Legends",
 	definition = function(object, x = unit(0.5, "npc"), y = unit(0.5, "npc"), just = "centre", test = FALSE) {
-	
+
 	legend = object@grob
 	legend = edit_vp_in_legend_grob(legend, x = x, y = y, valid.just = valid_just(just))
 
@@ -1084,6 +1386,7 @@ setMethod(f = "draw",
     }
 	grid.draw(legend)
 	if(test2) {
+		# grid.rect(width = grobWidth(legend), height = grobHeight(legend))
 		grid.text(test, x = 0.5, y = unit(1, "npc") - unit(1, "cm"))
 	}
 })
