@@ -8,6 +8,7 @@
 #      value representing whether the alteration is present or absent. 
 #      When the value is a list, the names of the list represent alteration types.
 #      You can use `unify_mat_list` to make all matrix having same row names and column names.
+# -name Name of the oncoPrint. Not necessary to specify.
 # -get_type If different alterations are encoded in the matrix as complex strings, this self-defined function
 #           determines how to extract them. It only works when ``mat`` is a matrix. The default value is `default_get_type`.
 # -alter_fun A single function or a list of functions which defines how to add graphics for different alterations.
@@ -53,7 +54,7 @@
 # == author
 # Zuguang Gu <z.gu@dkfz.de>
 #
-oncoPrint = function(mat, 
+oncoPrint = function(mat, name,
 	get_type = default_get_type,
 	alter_fun, 
 	alter_fun_is_vectorized = NULL,
@@ -134,7 +135,7 @@ oncoPrint = function(mat,
 
 		## check whether there are NA values in the matrix
 		if(any(is.na(mat))) {
-			message_wrap("Found NA values in the matrix and treat as no alteration. If `NA` means no alteration, you can explicitly set it to empty strings like ''. If `NA` is an alteration type, you should format it to a string like `'NA'` and define a graphic for it.")
+			message_wrap("Found NA values in the matrix and treat as no alteration. If `NA` means no alteration, you can explicitly set it to empty strings like ''. If `NA` is an alteration type, you should format it to a string like `'NA'` and define graphics for it.")
 		}
 
 		mat_list = lapply(all_type, function(type) {
@@ -172,11 +173,24 @@ oncoPrint = function(mat,
 	message_wrap(paste0("All mutation types: ", paste(all_type, collapse = ", "), "."))
 
 	# type as the third dimension
-	arr = array(FALSE, dim = c(dim(mat_list[[1]]), length(all_type)), dimnames = c(dimnames(mat_list[[1]]), list(all_type)))
+	if(is.null(dimnames(mat_list[[1]]))) {
+		dimnames = c(list(NULL), list(NULL), list(all_type))
+	} else {
+		dimnames = c(dimnames(mat_list[[1]]), list(all_type))
+	}
+	arr = array(FALSE, dim = c(dim(mat_list[[1]]), length(all_type)), dimnames = dimnames)
 	for(i in seq_along(all_type)) {
 		arr[, , i] = mat_list[[i]]
 	}
 
+	if(missing(name)) {
+        name = paste0("oncoPrint_", get_oncoprint_index() + 1)
+        increase_oncoprint_index()
+    } else if(is.null(name)) {
+        name = paste0("oncnPrint_", get_oncoprint_index() + 1)
+        increase_oncoprint_index()
+    }
+
 	oncoprint_row_order = function() {
 		order(rowSums(count_matrix), n_mut, decreasing = TRUE)
 	}
@@ -285,6 +299,8 @@ oncoPrint = function(mat,
 		if(alter_fun_is_vectorized) {
 			layer_fun = function(j, i, x, y, w, h, fill) {
 				v = pindex(arr, i, j)
+				v = as.vector(v)
+				names(v) = dimnames[[3]]
 				af(x, y, w, h, v, j, i)
 			}
 			cell_fun = NULL
@@ -292,6 +308,7 @@ oncoPrint = function(mat,
 			layer_fun = NULL
 			cell_fun = function(j, i, x, y, w, h, fill) {
 				v = arr[i, j, ]
+				names(v) = dimnames[[3]]
 				af(x, y, w, h, v, j, i)
 			}
 		}
@@ -392,6 +409,28 @@ oncoPrint = function(mat,
 	} else {
 		rn_ha = NULL
 	}
+
+	
+	if(!is.null(top_annotation)) {
+		if(inherits(top_annotation, "AnnotationFunction")) {
+	        stop_wrap("The annotation function `anno_*()` should be put inside `HeatmapAnnotation()`.")
+	    }
+	}
+	if(!is.null(bottom_annotation)) {
+	    if(inherits(bottom_annotation, "AnnotationFunction")) {
+	        stop_wrap("The annotation function `anno_*()` should be put inside `HeatmapAnnotation()`.")
+	    }
+	}
+	if(!is.null(left_annotation)) {
+		if(inherits(left_annotation, "AnnotationFunction")) {
+	        stop_wrap("The annotation function `anno_*()` should be put inside `rowAnnotation()`.")
+	    }
+	}
+	if(!is.null(right_annotation)) {
+	    if(inherits(right_annotation, "AnnotationFunction")) {
+	        stop_wrap("The annotation function `anno_*()` should be put inside `rowAnnotation()`.")
+	    }
+	}
 	
 	if(is.null(left_annotation)) {
 		if(pct_side == "left") {
@@ -441,13 +480,17 @@ oncoPrint = function(mat,
 	
 	#####################################################################
 	# the main matrix
-	if(length(col)) {
-		pheudo = c(names(col), rep(NA, nrow(arr)*ncol(arr) - length(col)))
-	} else {
-		pheudo = c("mutation", rep(NA, nrow(arr)*ncol(arr) - 1))
+	if(length(col) == 0) {
 		col = c("mutation" = "black")
 	}
 
+	pheudo = apply(arr, 1:2, function(x) {
+		if(all(!x)) {
+			return("")
+		} else {
+			paste(all_type[x], collapse = ";")
+		}
+	})
 	dim(pheudo) = dim(arr)[1:2]
 	dimnames(pheudo) = dimnames(arr)[1:2]
 	
@@ -531,7 +574,7 @@ oncoPrint = function(mat,
 		}
 	}
 
-	ht = Heatmap(pheudo, col = col, 
+	ht = Heatmap(pheudo, name = name, col = col, 
 		rect_gp = gpar(type = "none"), 
 		cluster_rows = cluster_rows, cluster_columns = cluster_columns, 
 		row_order = row_order, column_order = column_order,
@@ -668,6 +711,7 @@ unify_mat_list = function(mat_list, default = 0) {
 # -type A vector of the alteration types in the data. It can be a subset of all alteration types if you don't want to show them all.
 # -which Is it a row annotation or a column annotation?
 # -bar_width Width of the bars.
+# -beside Will bars be stacked or be positioned beside each other?
 # -ylim Data range.
 # -show_fraction Whether to show the numbers or the fractions?
 # -axis Whether draw axis?
@@ -683,7 +727,7 @@ unify_mat_list = function(mat_list, default = 0) {
 # Zuguang Gu <z.gu@dkfz.de>
 #
 anno_oncoprint_barplot = function(type = NULL, which = c("column", "row"),
-	bar_width = 0.6, ylim = NULL, show_fraction = FALSE, axis = TRUE,
+	bar_width = 0.6, beside = FALSE, ylim = NULL, show_fraction = FALSE, axis = TRUE,
 	axis_param = if(which == "column") default_axis_param("column") else list(side = "top", labels_rot = 0),
 	width = NULL, height = NULL, border = FALSE) {
 
@@ -718,7 +762,7 @@ anno_oncoprint_barplot = function(type = NULL, which = c("column", "row"),
 		v = v[, !is.na(col), drop = FALSE]
 		col = col[!is.na(col)]
 		fun = anno_barplot(v, gp = gpar(fill = col, col = NA), which = "column", ylim = ylim,
-			baseline = 0, height = anno_size$height, border = border, bar_width = bar_width,
+			baseline = 0, height = anno_size$height, border = border, bar_width = bar_width, beside = beside,
 			axis = axis, axis_param = axis_param)@fun
 		fun(index, k, n)
 	}
@@ -745,7 +789,7 @@ anno_oncoprint_barplot = function(type = NULL, which = c("column", "row"),
 		v = v[, !is.na(col), drop = FALSE]
 		col = col[!is.na(col)]
 		fun = anno_barplot(v, gp = gpar(fill = col, col = NA), which = "row", ylim = ylim,
-			baseline = 0, width = anno_size$width, border = border, bar_width = bar_width,
+			baseline = 0, width = anno_size$width, border = border, bar_width = bar_width, beside = beside,
 			axis = axis, axis_param = axis_param)@fun
 		fun(index, k, n)
 	}
@@ -762,7 +806,7 @@ anno_oncoprint_barplot = function(type = NULL, which = c("column", "row"),
 		which = which,
 		width = anno_size$width,
 		height = anno_size$height,
-		var_import = list(border, type, bar_width, axis, axis_param, anno_size, ylim, show_fraction)
+		var_import = list(border, type, bar_width, beside, axis, axis_param, anno_size, ylim, show_fraction)
 	)
 		
 	anno@subsetable = TRUE
