@@ -7,7 +7,7 @@
 #        The discrete colors sent to `grDevices::colorRampPalette` are also OK here. E.g.
 #        ``colorRampPalette(rev(brewer.pal(n = 7, name = "RdYlBu")))(100)`` can be simply 
 #        replaced as ``rev(brewer.pal(n = 7, name = "RdYlBu"))``.
-# -kmeans_k Not supported.
+# -kmeans_k The same as in `pheatmap::pheatmap`.
 # -breaks The same as in `pheatmap::pheatmap`.
 # -border_color The same as in `pheatmap::pheatmap`.
 # -cellwidth The same as in `pheatmap::pheatmap`.
@@ -55,21 +55,26 @@
 # -silent Not supported.
 # -na_col The same as in `pheatmap::pheatmap`.
 # -name Name of the heatmap. This argument is passed to `Heatmap`.
+# -heatmap_legend_param  Pass to `Heatmap`.
 # -... Other arguments passed to `Heatmap`.
+# -run_draw Whether to run ``draw()`` function to the heatmap object.
 #
 # == details
+# This function aims to execute ``pheatmap::pheatmap`` code purely with ComplexHeatmap.
+#
+# == seealso
 # See https://jokergoo.github.io/2020/05/06/translate-from-pheatmap-to-complexheatmap/
 #
+# `compare_pheatmap` that compares heatmaps between ``pheatmap::pheatmap()`` and ``ComplexHeatmap::pheatmap()``.
+#
 # == value
 # A `Heatmap-class` object.
 #
-# == seealso
-# `compare_pheatmap` that ompares heatmaps between ``pheatmap::pheatmap()`` and ``ComplexHeatmap::pheatmap()``.
 pheatmap = function(mat, 
     color = colorRampPalette(rev(brewer.pal(n = 7, name = "RdYlBu")))(100), 
     kmeans_k = NA, 
     breaks = NA, 
-    border_color = "grey60",
+    border_color = ifelse(nrow(mat) < 100 & ncol(mat) < 100, "grey60", NA),
     cellwidth = NA, 
     cellheight = NA, 
     scale = "none", 
@@ -114,16 +119,26 @@ pheatmap = function(mat,
     height = NA,
     silent = FALSE, 
     na_col = "#DDDDDD", 
-    name = "Matrix",
-    ...
-) {
+    name = NULL,
 
+    # argument specific for Heatmap()
+    heatmap_legend_param = list(),
+    ...,
+    run_draw = FALSE
+) {
   
     if(is.data.frame(mat)) {
         warning_wrap("The input is a data frame, convert it to the matrix.")
         mat = as.matrix(mat)
     }
 
+    if(!identical(kmeans_k, NA)) {
+        warning_wrap("argument `kmeans_k` is not suggested to use in pheatmap -> Heatmap translation because it changes the input matrix. You might check `row_km` and `column_km` arguments in Heatmap().")
+        km = kmeans(mat, centers = kmeans_k)
+        mat = km$centers
+        rownames(mat) = paste0("Cluster: ", seq_along(km$size), ", Size: ", km$size)
+    }
+
     if("row" %in% scale) {
 
         if(any(is.na(mat))) {
@@ -191,10 +206,6 @@ pheatmap = function(mat,
             }
         }
     }
-
-    if(!identical(kmeans_k, NA)) {
-        warning_wrap("argument `kmeans_k` is not supported in pheatmap -> Heatmap translation, skip it.")
-    }
     
     if(!identical(filename, NA)) {
         warning_wrap("argument `filename` is not supported in pheatmap -> Heatmap translation, skip it.")
@@ -214,6 +225,12 @@ pheatmap = function(mat,
 
     ht_param$rect_gp = gpar(col = border_color)
 
+    if(nrow(mat) > 1000 || ncol(mat) > 1000) {
+        if(!is.na(border_color)) {
+            warning_wrap("border color is set for the matrix with large numbers of rows or columns. You might only be able to see the border colors in the plot. Set `border_color = NA` to get rid of it.")
+        }
+    }
+
     if(!identical(cellwidth, NA)) {
         ht_param$width = ncol(mat)*unit(cellwidth, "pt")
     }
@@ -251,7 +268,7 @@ pheatmap = function(mat,
     ht_param$column_dend_height = unit(treeheight_col, "pt")
 
     ht_param$show_heatmap_legend = legend
-    heatmap_legend_param = list()
+
     if(identical(scale, "row") || identical(scale, "column")) {
         if(identical(legend_breaks, NA)) {
             lim = quantile(abs(mat), 0.975)
@@ -441,17 +458,24 @@ pheatmap = function(mat,
         ht_param$column_title_gp = gpar(fontface = "bold", fontsize = 1.3*fontsize)
     }
     ht_param = c(ht_param, list(...))
-    do.call(Heatmap, ht_param)
+    ht = do.call(Heatmap, ht_param)
+    attr(ht, "translate_from") = "pheatmap"
+
+    if(run_draw) {
+        draw(ht)
+    } else {
+        ht
+    }
 }
 
 # == title
 # Compare heatmaps between pheatmap::pheatmap() and ComplexHeatmap::pheatmap()
 #
 # == param
-# -... The same set of rguments passed to `pheatmap::pheatmap` and `pheatmap`.
+# -... The same set of arguments passed to ``pheatmap::pheatmap`` and ``ComplexHeatmap::pheatmap``.
 #
 # == details
-# The function plots two heatmaps, one by `pheatmap::pheatmap` and one by `pheatmap`.
+# The function plots two heatmaps, one by ``pheatmap::pheatmap`` and one by ``ComplexHeatmap::pheatmap``.
 # Users can see the difference between the two implementations.
 #
 # == example
