@@ -37,6 +37,25 @@ anno = AnnotationFunction(fun = fun, var_import = "x")
 anno = AnnotationFunction(fun = fun, var_import = list(x))
 
 
+x = runif(10)
+cell_fun = function(i) {
+	pushViewport(viewport(yscale = c(0, 1)))
+	grid.points(unit(0.5, "npc"), x[i])
+	popViewport()
+}
+anno = AnnotationFunction(cell_fun = cell_fun, var_import = "x")
+ha = HeatmapAnnotation(foo = anno)
+draw(ha, 1:10, test = T)
+
+cell_fun = function(i) {
+	pushViewport(viewport(xscale = c(0, 1)))
+	grid.points(x[i], unit(0.5, "npc"))
+	popViewport()
+}
+anno = AnnotationFunction(cell_fun = cell_fun, var_import = "x", which = "row")
+ha = rowAnnotation(foo = anno)
+draw(ha, 1:10, test = T)
+
 # devAskNewPage(ask = dev.interactive())
 
 ########### testing anno_simple ############
@@ -243,6 +262,16 @@ anno = anno_barplot(matrix(nc = 2, c(1:10, 10:1)), which = "row", axis_param = l
 draw(anno, test = "a vector")
 
 
+anno = anno_barplot(matrix(nc = 2, c(1:10, 10:1)), beside = TRUE)
+draw(anno, test = "a matrix")
+draw(anno[1:5], test = "a matrix, subset")
+anno = anno_barplot(matrix(nc = 2, c(1:10, 10:1)), beside = TRUE, which = "row")
+draw(anno, test = "a matrix, on rows")
+anno = anno_barplot(matrix(nc = 2, c(1:10, 10:1)), beside = TRUE, gp = gpar(fill = 2:3, col = 2:3))
+draw(anno, test = "a matrix with fill")
+
+
+
 ##### test anno_boxplot #####
 set.seed(123)
 m = matrix(rnorm(100), 10)
@@ -630,3 +659,19 @@ dev.new(width = 3.938326, height = 4.502203)
 draw(Heatmap(mat, right_annotation = rowAnnotation(foo = anno), 
 	row_split = row_split))
 
+
+
+#### anno_custome ###
+x = sort(sample(letters[1:3], 10, replace = TRUE))
+graphics = list(
+	"a" = function(x, y, w, h) grid.points(x, y, pch = 16),
+	"b" = function(x, y, w, h) grid.rect(x, y, w*0.8, h*0.8, gp = gpar(fill = "red")),
+	"c" = function(x, y, w, h) grid.segments(x - 0.5*w, y - 0.5*h, x + 0.5*w, y + 0.5*h, gp = gpar(lty = 2))
+)
+
+anno = anno_customize(x, graphics = graphics)
+draw(anno, index = 1:10, test = "")
+
+anno = anno_customize(c(x, "d"), graphics = graphics)
+
+
